import java.awt.*;
/**
  *
  * Klasse zur Simulation einer Spielposition beim 3d-Viergewinnt.
  *
  * @version 1.0 vom 17.11.2008
  * @author Garmann
  */

public class Position {

  // Anfang Variablen
  private int amZug;
  private int[][][] feld;
  private int[][] reihenanzahl = new int[2][5];  //[Spieler][Steine]

  // Ende Variablen

  public Position() {
    feld = new int[4][4][4];
    amZug = 0;
  }

  public Position(int amZug) {
    this();
    this.amZug = amZug;
  }

  // Anfang Ereignisprozeduren
  public int amZug() {
    return amZug;
  }


  public void zugAusfuehren(Zug zug) {
    int z = 4;
    while (z>0 && feld[zug.gibX()][zug.gibY()][z-1] == 0) z--;
    feld[zug.gibX()][zug.gibY()][z] = amZug*4+1;
    amZug = 1 - amZug;
  }

  public boolean zulaessig(Zug zug) {
    return (feld[zug.gibX()][zug.gibY()][3] == 0);
  }

  public void zugZueruecknehmen(Zug zug) {
    int z = 3;
    while (z>=0 && feld[zug.gibX()][zug.gibY()][z] == 0) z--;
    feld[zug.gibX()][zug.gibY()][z] = 0;
    amZug = 1 - amZug;
  }

  public boolean spielendeErreicht() {
    berechneReihenanzahl();
    
    return (reihenanzahl[0][4]>0 || reihenanzahl[1][4]>0) ||
           (reihenanzahl[0][4]+reihenanzahl[0][3]+reihenanzahl[0][2]+reihenanzahl[0][1]+reihenanzahl[0][0]==0) ||
           (reihenanzahl[1][4]+reihenanzahl[1][3]+reihenanzahl[1][2]+reihenanzahl[1][1]+reihenanzahl[1][0]==0);
  }


  public void berechneReihenanzahl() {

    for (int anzahl=0; anzahl <=4; anzahl++) {
      for (int spielerNr = 0; spielerNr < 2; spielerNr++) {
        reihenanzahl[spielerNr][anzahl] = 0;
        
        for (int z = 0; z < 4; z++) {
          for (int y = 0; y < 4; y++) {
            if (feld[0][y][z] + feld[1][y][z] + feld[2][y][z] + feld[3][y][z] == (spielerNr*4+1)*anzahl){
              reihenanzahl[spielerNr][anzahl]++;
            }
          }
        }

        for (int y = 0; y < 4; y++) {
          for (int x = 0; x < 4; x++) {
            if (feld[x][y][0] + feld[x][y][1] + feld[x][y][2] + feld[x][y][3] == (spielerNr*4+1)*anzahl){
              reihenanzahl[spielerNr][anzahl]++;
            }
          }
        }

        for (int z = 0; z < 4; z++) {
          for (int x = 0; x < 4; x++) {
            if (feld[x][0][z] + feld[x][1][z] + feld[x][2][z] + feld[x][3][z] == (spielerNr*4+1)*anzahl){
              reihenanzahl[spielerNr][anzahl]++;
            }
          }
        }

        for (int z = 0; z < 4; z++) {
          if (feld[0][0][z] + feld[1][1][z] + feld[2][2][z] + feld[3][3][z] == (spielerNr*4+1)*anzahl){
            reihenanzahl[spielerNr][anzahl]++;
          }
          if (feld[0][3][z] + feld[1][2][z] + feld[2][1][z] + feld[3][0][z] == (spielerNr*4+1)*anzahl){
            reihenanzahl[spielerNr][anzahl]++;
          }
        }

        for (int y = 0; y < 4; y++) {
          if (feld[0][y][0] + feld[1][y][1] + feld[2][y][2] + feld[3][y][3] == (spielerNr*4+1)*anzahl){
            reihenanzahl[spielerNr][anzahl]++;
          }
          if (feld[0][y][3] + feld[1][y][2] + feld[2][y][1] + feld[3][y][0] == (spielerNr*4+1)*anzahl){
            reihenanzahl[spielerNr][anzahl]++;
          }
        }

        for (int x = 0; x < 4; x++) {
          if (feld[x][0][0] + feld[x][1][1] + feld[x][2][2] + feld[x][3][3] == (spielerNr*4+1)*anzahl){
            reihenanzahl[spielerNr][anzahl]++;
          }
          if (feld[x][0][3] + feld[x][1][2] + feld[x][2][1] + feld[x][3][0] == (spielerNr*4+1)*anzahl){
            reihenanzahl[spielerNr][anzahl]++;
          }
        }

        if (feld[0][0][0] + feld[1][1][1] + feld[2][2][2] + feld[3][3][3] == (spielerNr*4+1)*anzahl){
          reihenanzahl[spielerNr][anzahl]++;
        }
        if (feld[0][0][3] + feld[1][1][2] + feld[2][2][1] + feld[3][3][0] == (spielerNr*4+1)*anzahl){
          reihenanzahl[spielerNr][anzahl]++;
        }
        if (feld[0][3][0] + feld[1][2][1] + feld[2][1][2] + feld[3][0][3] == (spielerNr*4+1)*anzahl){
          reihenanzahl[spielerNr][anzahl]++;
        }
        if (feld[3][0][0] + feld[2][1][1] + feld[1][2][2] + feld[0][3][3] == (spielerNr*4+1)*anzahl){
          reihenanzahl[spielerNr][anzahl]++;
        }

      }

//      System.out.println("  "+anzahl+": "+reihenanzahl[0][anzahl]+"\t"+reihenanzahl[1][anzahl]);
    }
  }

  public int[][] reihenanzahl() {
    berechneReihenanzahl();
    return reihenanzahl;
  }
  
  public void ausgeben(Container cp) {
    Graphics g = cp.getGraphics();
//    g.setColor(Color.white);
//    g.fillRect(0,0,cp.getWidth(),cp.getHeight());
    for (int x = 0; x < 4; x++) {
      for (int y = 0; y < 4; y++) {
        g.drawLine(50+x*80+y*20+10,300-y*20+10,
                   50+x*80+y*20+10,300-y*20+10-60);
        for (int z = 0; z < 4; z++) {
          switch (feld[x][y][z]) {
            case 0: g.setColor(Color.gray);
                    break;
            case 1: g.setColor(Color.red);
                    break;
            case 5: g.setColor(Color.blue);
                    break;
          }
          if (feld[x][y][z]>0)
            g.fillOval(50+x*80+y*20+y,300-y*20-z*20+y,20-y*2,20-y*2);
        }
      }
    }
  }
  // Ende Ereignisprozeduren
}

